#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Feb 12 14:55:37 2024

@author: eemili
"""

import os
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.lines as lines
import matplotlib.dates as mdates
from datetime import datetime as dt
from datetime import timedelta as td

# inidate = '2024011300'
# step = td(hours=24)
# nsteps = 3
# dateplt = (dt.strptime('2024011300', '%Y%m%d%H'),dt.strptime('2024011600', '%Y%m%d%H'))
# minobs = 250

inidate = '2024070300'
step = td(hours=24)
nsteps = 28
dateplt = (dt.strptime('2024070300', '%Y%m%d%H'),dt.strptime('2024080100', '%Y%m%d%H'))
minobs = 5000

# inidate = '2024101100'
# step = td(hours=24)
# nsteps = 9
# dateplt = (dt.strptime('2024101300', '%Y%m%d%H'),dt.strptime('2024102000', '%Y%m%d%H'))
# minobs = 5000

explist, exptype, expcolor = [], [], []
# explist.append('/esarchive/exp/monarch/a5zv')
# exptype.append('OPER-FCST')
# expcolor.append(['navy','royalblue','lightsteelblue'])
# explist.append('/esarchive/exp/monarch/a6ub')
# exptype.append('DA')
# expcolor.append(['saddlebrown','chocolate','sandybrown'])
# explist.append('/esarchive/exp/monarch/a7o0')
# exptype.append('DA')
# expcolor.append(['hotpink','crimson','plum'])
# explist.append('/esarchive/exp/monarch/a7rj')
# exptype.append('INI-FCST')
# expcolor.append(['saddlebrown','chocolate','sandybrown'])

# Operational DA
explist.append('/esarchive/scratch/eemili/a00c_cirrus')
exptype.append('OPER-FCST')
expcolor.append(['navy','royalblue','lightsteelblue'])
explist.append('/esarchive/exp/monarch//a003_cirrus')
exptype.append('OPER-FCST')
expcolor.append(['saddlebrown','chocolate','sandybrown'])
# explist.append('/esarchive/scratch/eemili/a00g_cirrus')
# exptype.append('DA')
# expcolor.append(['hotpink','crimson','plum'])
explist.append('/esarchive/exp/monarch/a7pb')
exptype.append('DA')
expcolor.append(['hotpink','crimson','plum'])
# explist.append('/esarchive/scratch/eemili/a00f_cirrus')
# exptype.append('INIT-FCST')
# expcolor.append(['saddlebrown','chocolate','sandybrown','bisque'])

# Hardcoded legends
# explegends = [['FIRST GUESS D0','ANALYSIS D0'],['FIRST GUESS D0','ANALYSIS D0']]
# explegends = [['OPER-FCST D1','OPER-FCST D2','OPER-FCST D3'],['FIRST GUESS D0','ANALYSIS D0'],['DA-FCST D1','DA-FCST D2','DA-FCST D3']]
# explegends = [['FIRST GUESS D0','ANALYSIS D0'],['DA-FCST D1','DA-FCST D2','DA-FCST D3']]
# explegends = [['OPER D1','OPER D2','OPER D3'],['DA-FCST D1','DA-FCST D2','DA-FCST D3']]
# explegends = [['OPER D1','OPER D2','OPER D3'],['FIRST GUESS D0','ANALYSIS D0']]
explegends = [['OPER D1','OPER D2','OPER D3'],['OPER D1','OPER D2','OPER D3'],['FIRST GUESS D0','ANALYSIS D0']]

markers = ['o', 'diamond', 'thin_diamond']
msizes = [1 , 0.6, 0.4, 0.2]

outdir = '/esarchive/scratch/eemili/oper-da/monitoring'

if not os.path.exists(outdir): os.makedirs(outdir)

inidate = dt.strptime(inidate, '%Y%m%d%H')
dates = [inidate+i*step for i in range(nsteps)]

fig = plt.figure(figsize=(40,30),dpi=250)
# fig = plt.figure(figsize=(16,5),dpi=300)
fig.autofmt_xdate()
hours = mdates.HourLocator(interval=12)

axnobs = fig.add_subplot(411)
axnobs.set_title("Number of %s observations" % ('VIIRS'))
    
axnobs.grid(True)   
axnobs.axis(xmin=dateplt[0],xmax=dateplt[1])
axnobs.set_xlabel('Time (UTC)')
axnobs.xaxis.set_major_formatter(mdates.DateFormatter('%m-%d'))
axnobs.xaxis.set_minor_locator(hours)
axnobs.xaxis.set_minor_formatter(mdates.DateFormatter('%H'))

axobs = fig.add_subplot(412)
axobs.set_title("%s statistics (%s)" % ('DOD average','unitless'))
axobs.set_ylabel("%s" % ('unitless'))
                     
axobs.grid(True)   
axobs.axis(xmin=dateplt[0],xmax=dateplt[1])
axobs.set_xlabel('Time (UTC)')
axobs.xaxis.set_major_formatter(mdates.DateFormatter('%m-%d'))
axobs.xaxis.set_minor_locator(hours)
axobs.xaxis.set_minor_formatter(mdates.DateFormatter('%H'))

axdep = fig.add_subplot(413)
# axdep = fig.add_subplot(111)
axdep.set_title("%s (%s)" % ('Bias','unitless'))
axdep.set_ylabel("%s" % ('unitless'))
    
axdep.grid(True)
axdep.axis(xmin=dateplt[0],xmax=dateplt[1])
axdep.set_xlabel('Time (UTC)')
axdep.xaxis.set_major_formatter(mdates.DateFormatter('%m-%d'))
axdep.xaxis.set_minor_locator(hours)
axdep.xaxis.set_minor_formatter(mdates.DateFormatter('%H'))

axstd = fig.add_subplot(414)
axstd.set_title("%s (%s)" % ('STD','unitless'))
axstd.set_ylabel("%s" % ('unitless'))
    
axstd.grid(True)
axstd.axis(xmin=dateplt[0],xmax=dateplt[1])
axstd.set_xlabel('Time (UTC)')
axstd.xaxis.set_major_formatter(mdates.DateFormatter('%m-%d'))
axstd.xaxis.set_minor_locator(hours)
axstd.xaxis.set_minor_formatter(mdates.DateFormatter('%H'))

expstyles = np.zeros(len(explist),dtype=np.int8)

for idate,date in enumerate(dates):
    
    for iexp,exp in enumerate(explist):
        
        npfile = os.path.join(explist[iexp],'plots',date.strftime('%Y%m%d'),
                              'VIIRS_%s_time_series.npz' % (date.strftime('%Y%m%d')))
        
        if os.path.exists(npfile):
            
            d = np.load(npfile,allow_pickle=True)
            
            # Use different styles for each day of forecast
            leadtime = d['time']-d['time'][0]
            styleidx = np.array([int(l.total_seconds()/step.total_seconds()) for l in leadtime])
            
            if 'da' in exptype[iexp].lower():
                
                expstyles[iexp] = 2
                
                axnobs.plot(d['time'],d['nrejobs'],color='black',
                    ls='--',lw=2,marker=None,label='Rejected',markersize=3.0)
                axnobs.plot(d['time'],d['ntotobs'],color='black',
                        ls='-',lw=2,marker=None,label='Total',markersize=4.0)
                
                val = np.ma.masked_values(d['y_avg'],d['missing_value'])
                axobs.errorbar(d['time'],np.ma.masked_where(d['nusedobs'] < minobs,val),
                            yerr=np.ma.masked_values(d['y_std'],d['missing_value']).filled(0.0),color='black',
                        ls=None,marker='o',label='y_da',markersize=4.0,
                        capsize=6, elinewidth=1.7, markeredgewidth=1.7)
                
                val = np.ma.masked_values(d['hofxfg_avg'],d['missing_value'])
                axobs.errorbar(d['time'],np.ma.masked_where(d['nusedobs'] < minobs,val),
                            yerr=np.ma.masked_values(d['hofxfg_std'],d['missing_value']).filled(0.0),color=expcolor[iexp][0],
                        ls='-',marker='o',label='hofxfg',markersize=4.0,
                        capsize=6, elinewidth=1.7, markeredgewidth=1.7)
                val = np.ma.masked_values(d['hofxan_avg'],d['missing_value'])
                axobs.errorbar(d['time'],np.ma.masked_where(d['nusedobs'] < minobs,val),
                            yerr=np.ma.masked_values(d['hofxan_std'],d['missing_value']).filled(0.0),color=expcolor[iexp][1],
                        ls=None,marker='o',label='hofxan',markersize=4.0,
                        capsize=4, elinewidth=1.3, markeredgewidth=1.3)
                
                val = np.ma.masked_values(d['fgdep_avg'],d['missing_value'])
                axdep.plot(d['time'],np.ma.masked_where(d['nusedobs'] < minobs,val),
                            color=expcolor[iexp][0],
                        ls='-',marker='o',label='fgdep',markersize=4.0)
                val = np.ma.masked_values(d['fgdep_std'],d['missing_value'])
                axstd.plot(d['time'],np.ma.masked_where(d['nusedobs'] < minobs,val),
                            color=expcolor[iexp][0],
                        ls='-',marker='o',label='fgdep',markersize=4.0)
                val = np.ma.masked_values(d['andep_avg'],d['missing_value'])
                axdep.plot(d['time'],np.ma.masked_where(d['nusedobs'] < minobs,val),
                           color=expcolor[iexp][1],
                        ls=None,marker='o',label='andep',markersize=4.0)
                val = np.ma.masked_values(d['andep_std'],d['missing_value'])
                axstd.errorbar(d['time'],np.ma.masked_where(d['nusedobs'] < minobs,val),
                                color=expcolor[iexp][1],
                        ls=None,marker='o',label='andep',markersize=4.0)
                
            if 'fc' in exptype[iexp].lower():
                
                expstyles[iexp] = np.unique(styleidx).size
                
                axnobs.plot(d['time'],d['nobs'],color=expcolor[iexp][0],
                        ls='--',lw=1,marker=None,label='Forecast',markersize=4.0)
                
                val = np.ma.masked_values(d['y_avg'],d['missing_value'])
                axobs.errorbar(d['time'],np.ma.masked_where(d['nobs'] < minobs,val),
                            yerr=np.ma.masked_values(d['y_std'],d['missing_value']).filled(0.0),color='black',
                        ls='--',marker='o',label='y_fc',markersize=4.0,
                        capsize=4, elinewidth=1.3, markeredgewidth=1.3)
                
                for ist,style in enumerate(np.unique(styleidx)):
                    
                    time = d['time'][styleidx == ist]
                    nobs = d['nobs'][styleidx == ist]
                    val = np.ma.masked_values(d['hofxfc_avg'],d['missing_value'])[styleidx == ist]
                    err = np.ma.masked_values(d['hofxfc_std'],d['missing_value']).filled(0.0)[styleidx == ist]
                    axobs.errorbar(time,np.ma.masked_where(nobs < minobs,val),
                            yerr=err,color=expcolor[iexp][ist],
                            ls=None,marker='o',label='hofxfc',markersize=4.0*msizes[ist],
                            capsize=6*msizes[ist], elinewidth=1.7*msizes[ist], markeredgewidth=1.7*msizes[ist])
                    
                    val = np.ma.masked_values(d['fcdep_avg'],d['missing_value'])[styleidx == ist]
                    axdep.plot(time,np.ma.masked_where(nobs < minobs,val),
                            color=expcolor[iexp][ist],
                            ls=None,marker='o',label='fcdep',markersize=4.0*msizes[ist])
                    
                    val = np.ma.masked_values(d['fcdep_std'],d['missing_value'])[styleidx == ist]
                    axstd.plot(time,np.ma.masked_where(nobs < minobs,val),
                            color=expcolor[iexp][ist],
                            ls=None,marker='o',label='fcdep',markersize=4.0*msizes[ist])
                    
handles = []

for iexp,exp in enumerate(explist):
    for istyle in range(expstyles[iexp]):
        #handles.append(lines.Line2D([0],[0],color=expcolor[iexp][istyle],lw=3,label=exptype[iexp]+'+%i' % (istyle)))
        handles.append(lines.Line2D([0],[0],color=expcolor[iexp][istyle],lw=3,label=explegends[iexp][istyle]))
axdep.legend(handles=handles)
# axstd.legend(handles=handles)
# handles.append(lines.Line2D([0],[0],color='black',lw=3 , label='DA Obs'))
handles.append(lines.Line2D([0],[0],color='black',lw=2 , ls='--', label='VIIRS Obs'))
# axobs.legend(handles=handles)

handles = []
# handles.append(lines.Line2D([0],[0],color='black',lw=3 , label='DA Used Obs'))
# handles.append(lines.Line2D([0],[0],color='black',lw=3 ,ls='--', label='DA Rejected Obs'))
handles.append(lines.Line2D([0],[0],color='black',lw=1 , ls='--', label='VIIRS Obs'))
# axnobs.legend(handles=handles)                    

prefixname = 'oper_'
for exp in explist: prefixname += exp.split('/')[-1]+'_'
plt.savefig(os.path.join(outdir,'%sVIIRS_time_series.png' % (prefixname)),format='png')
plt.close(fig)