#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Feb  9 15:14:11 2024

@author: eemili
"""

# import matplotlib
# matplotlib.use('Agg')
import matplotlib.pyplot as plt
import numpy as np
import matplotlib.colors as mc
import cartopy.crs as ccrs
from mpl_toolkits.axes_grid1 import make_axes_locatable

def plot2Dmap(field,lats,lons,**kwargs):
    
    filename = 'fig.png'
    # colormap = 'viridis'
    colormap = 'YlOrBr'
    if 'filename' in kwargs: filename = kwargs['filename']
    if 'colormap' in kwargs: colormap = kwargs['colormap']
    # if not os.path.exists(os.path.split(filename)[0]):
    #     os.makedirs(os.path.split(filename)[0])
    
    fig = plt.figure(dpi=250)

    # Setup a global EckertIII map with faint coastlines.
    ax = fig.add_subplot(1, 1, 1, projection=ccrs.PlateCarree())
    # ax.set_global()
    if 'plotregion' in kwargs:
        plotregion = kwargs['plotregion']
        ax.set_extent([plotregion['LonMin'], plotregion['LonMax'], 
                       plotregion['LatMin'], plotregion['LatMax']], crs=ccrs.PlateCarree())
        
    ax.coastlines('110m', alpha=0.1)
    
    vmin, vmax = field.min(), field.max()
    if 'range' in kwargs and kwargs['range'] is not None:
        if "min" in kwargs['range']: vmin = kwargs['range']["min"]
        if "max" in kwargs['range']: vmax = kwargs['range']["max"]
        
    levels=np.linspace(vmin,vmax,21)
    norm = mc.BoundaryNorm(levels, 256)
    filled_c = ax.contourf(lons, lats, field, 
                           transform=ccrs.PlateCarree(),
                           cmap=colormap,
                           levels=levels,
                           norm=norm,
                           extend='both')
    
    # And black line contours.
    # line_c = ax.contour(lons, lats, field, levels=filled_c.levels,
    #                 colors=['black'],
    #                 transform=ccrs.PlateCarree())

    # Uncomment to make the line contours invisible.
    # plt.setp(line_c.collections, visible=False)
    
    # Add a colorbar for the filled contour.
    cbar = fig.colorbar(filled_c, orientation='horizontal')

    # Use the line contours to place contour labels.
    # ax.clabel(
    #     line_c,  # Typically best results when labelling line contours.
    #     colors=['black'],
    #     manual=False,  # Automatic placement vs manual placement.
    #     inline=True,  # Cut the line where the label will be placed.
    #     fmt=' {:.0f} '.format,  # Labes as integers, with some extra space.
    # )
    
    if 'title' in kwargs: ax.set_title(kwargs['title'])

    if 'show' in kwargs and kwargs['show']: 
        plt.show()
    else:
        plt.savefig(filename)
    plt.close(fig)
    
def plotobsloc(lats,lons,**kwargs):
    
    fig = plt.figure(figsize=(12,8),dpi=250)

    # Setup a global EckertIII map with faint coastlines.
    ax = fig.add_subplot(1, 1, 1, projection=ccrs.PlateCarree())
    
    if 'plotregion' in kwargs:
        plotregion = kwargs['plotregion']
        ax.set_extent([plotregion['LonMin'], plotregion['LonMax'], 
                       plotregion['LatMin'], plotregion['LatMax']], crs=ccrs.PlateCarree())
    
    ax.coastlines('110m', alpha=0.1)
    ax.scatter(lons, lats, color='black', marker='.', s=5, transform=ccrs.PlateCarree())
    
    plt.show()
    plt.close(fig)
    
def plotobsval(val,lats,lons,**kwargs):
    
    # filename = 'fig.png'
    colormap = 'viridis'
    # colormap = 'YlOrBr'
    if 'filename' in kwargs: filename = kwargs['filename']
    if 'colormap' in kwargs: colormap = kwargs['colormap']
    
    fig = plt.figure(figsize=(12,8),dpi=250)

    # Setup a global EckertIII map with faint coastlines.
    ax = fig.add_subplot(1, 1, 1, projection=ccrs.PlateCarree())
    
    if 'plotregion' in kwargs:
        plotregion = kwargs['plotregion']
        ax.set_extent([plotregion['LonMin'], plotregion['LonMax'], 
                       plotregion['LatMin'], plotregion['LatMax']], crs=ccrs.PlateCarree())
    
    ax.coastlines('110m', alpha=0.1)
    
    vmin, vmax = val.min(), val.max()
    if 'range' in kwargs and kwargs['range'] is not None:
        if "min" in kwargs['range']: vmin = kwargs['range']["min"]
        if "max" in kwargs['range']: vmax = kwargs['range']["max"]
    
    # levels=np.linspace(vmin,vmax,11)
    #norm = mc.BoundaryNorm(levels, 256)
    
    im = ax.scatter(lons, lats, c=val, cmap=colormap, marker='.', s=5, 
               vmin=vmin, vmax=vmax,
               transform=ccrs.PlateCarree())
    
    label = 'Visibility [m]'
    if 'label' in kwargs:
        label = kwargs['label']
    # Add a colorbar
    cb = plt.colorbar(im, ax=ax, label=label, orientation="horizontal", extend='both', shrink=0.8)
    cb.ax.tick_params(labelsize=10)
    plt.show()
    plt.close(fig)
    
def plotobshisto(val,**kwargs):
    
    # filename = 'fig.png'
    if 'filename' in kwargs: filename = kwargs['filename']
    
    fig = plt.figure(figsize=(6,4),dpi=250)
    
    ax = fig.add_subplot(111)
    vmin, vmax = val.min(), val.max()
    if 'range' in kwargs and kwargs['range'] is not None:
        if "min" in kwargs['range']: vmin = kwargs['range']["min"]
        if "max" in kwargs['range']: vmax = kwargs['range']["max"]
    
    # levels=np.linspace(vmin,vmax,11)
    #norm = mc.BoundaryNorm(levels, 256)
    ax.hist(val,bins=15)
    ax.set_xlim((vmin,vmax))

    plt.show()
    plt.close(fig)